@@ -3,7 +3,9 @@ __license__ = "Apache-2.0"
 
 import inspect
 from functools import wraps
-from typing import Callable, Iterator
+
+from typing import Any, Dict, Callable, Tuple, Iterable, Iterator
+
 
 import ruamel.yaml.constructor
 
@@ -79,40 +81,15 @@ class BaseDriver(metaclass=DriverType):
     A :class:`BaseDriver` needs to be :attr:`attached` to a :class:`jina.peapods.pea.BasePea` before using. This is done by
     :func:`attach`. Note that a deserialized :class:`BaseDriver` from file is always unattached.
 
-    .. note::
-        ``if`` accepts a single expression. It can be used for filter doc and chunk
-
-        Usage in YAML:
-
-        .. highlight:: yaml
-        .. code-block:: yaml
-
-            !Encode2
-            requests:
-              on:
-                IndexRequest:
-                  - !EncodeDriver
-                    if: doc.mime_type.startswith('image')
-
-        Usage in Python
-
-        .. highlight:: python
-        .. code-block:: python
-
-            class FilterDriver(PruneDriver):
-                if_expression = 'doc.doc_id % 2 ==0'
-
     """
 
     store_args_kwargs = False  #: set this to ``True`` to save ``args`` (in a list) and ``kwargs`` (in a map) in YAML config
-    if_expression = ''
 
     def __init__(self, *args, **kwargs):
         self.attached = False  #: represent if this driver is attached to a :class:`jina.peapods.pea.BasePea` (& :class:`jina.executors.BaseExecutor`)
         self.pea = None  # type: 'BasePea'
-        self._compiled_if = None
 
-    def attach(self, pea: 'BasePea', *args, **kwargs):
+    def attach(self, pea: 'BasePea', *args, **kwargs) -> None:
         """Attach this driver to a :class:`jina.peapods.pea.BasePea`
 
         :param pea: the pea to be attached.
@@ -120,43 +97,6 @@ class BaseDriver(metaclass=DriverType):
         self.pea = pea
         self.attached = True
 
-    @property
-    def compiled_if(self):
-        """Get the bytecode of the if expression, faster for :func:`eval` """
-        if self._compiled_if:
-            return self._compiled_if
-        elif self.if_expression:
-            self._compiled_if = compile(self.if_expression, '<string>', 'eval')
-            return self._compiled_if
-
-    def _test_eval_if(self):
-        return eval(self.compiled_if)
-
-    @property
-    def docs(self):
-        """Get all docs in the request, filtered by ``if`` when the expression contains ``doc``
-
-        Always use ``self.docs`` instead of ``self.req.docs`` to access filtered docs
-        """
-        if 'doc' not in self.if_expression:
-            yield from self.req.docs
-        else:
-            for doc in self.req.docs:
-                if eval(self.compiled_if):
-                    yield doc
-
-    def chunks(self, d) -> Iterator['jina_pb2.Chunk']:
-        """Get all chunks in the document ``d``, filtered by ``if`` when the expression contains ``chunk``
-
-        Always use ``self.chunks(d)`` instead of ``d.chunks`` to access filtered chunks
-        """
-        if 'chunk' not in self.if_expression:
-            yield from d.chunks
-        else:
-            for chunk in d.chunks:
-                if eval(self.compiled_if):
-                    yield chunk
-
     @property
     def req(self) -> 'jina_pb2.Request':
         """Get the current request, shortcut to ``self.pea.request``"""
@@ -167,6 +107,13 @@ class BaseDriver(metaclass=DriverType):
         """Get the current request, shortcut to ``self.pea.message``"""
         return self.pea.message
 
+    @property
+    def queryset(self) -> Iterator['jina_pb2.QueryLang']:
+        if self.pea and self.msg:
+            return self.pea.message.request.queryset
+        else:
+            return []
+
     @property
     def envelope(self) -> 'jina_pb2.Envelope':
         """Get the current request, shortcut to ``self.pea.message``"""
@@ -187,8 +134,6 @@ class BaseDriver(metaclass=DriverType):
         r = {}
         if a:
             r['with'] = a
-        if getattr(data, 'if_expression'):
-            r['if'] = data.if_expression
         return r
 
     @classmethod
@@ -208,14 +153,12 @@ class BaseDriver(metaclass=DriverType):
             constructor, node, deep=True)
 
         obj = cls(**data.get('with', {}))
-        # priority YAML if > class member
-        obj.if_expression = data.get('if', getattr(obj, 'if_expression', ''))
         return obj
 
     def __eq__(self, other):
         return self.__class__ == other.__class__
 
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         """Do not save the BasePea, as it would be cross-referencing. In other words, a deserialized :class:`BaseDriver` from
         file is always unattached. """
         d = dict(self.__dict__)
@@ -225,7 +168,100 @@ class BaseDriver(metaclass=DriverType):
         return d
 
 
-class BaseExecutableDriver(BaseDriver):
+class BaseRecursiveDriver(BaseDriver):
+
+    def __init__(self, depth_range: Tuple[int] = (0, 0), apply_order: str = 'post',
+                 traverse_on: Tuple[str] = ('chunks',), *args, **kwargs):
+        """
+
+        :param depth_range: right-exclusive range of the recursion depth, (0,0) for root-level only
+        :param apply_order: the traverse and apply order. if 'post' then first traverse then call apply, if 'pre' then first apply then traverse
+        :param args:
+        :param kwargs:
+        """
+        super().__init__(*args, **kwargs)
+        self._depth_start = depth_range[0]
+        self._depth_end = depth_range[1]
+        if isinstance(traverse_on, str):
+            traverse_on = (traverse_on,)
+        self.traverse_fields = set(traverse_on)
+        if apply_order in {'post', 'pre'}:
+            self.recursion_order = apply_order
+        else:
+            raise AttributeError('can only accept oder={"pre", "post"}')
+        self.is_apply = True
+        self.is_apply_all = True
+
+    def _apply(self, doc: 'jina_pb2.Document', context_doc: 'jina_pb2.Document', field: str, *args, **kwargs):
+        """ Apply function works on each doc, one by one, modify the doc in-place
+
+        :param doc: the :class:`jina_pb2.Document` object to work on. It could come from ``matches``/``chunks``.
+        :param context_doc: the owner of ``doc``
+        :param field: where ``doc`` comes from, either ``matches`` or ``chunks``
+        """
+
+    def _apply_all(self, docs: Iterable['jina_pb2.Document'], context_doc: 'jina_pb2.Document', field: str, *args,
+                   **kwargs):
+        """ Apply function works on a list of docs, modify the docs in-place
+
+        Depending on the value of ``order`` of :class:`BaseRecursiveDriver`, :meth:`apply_all` applies before or after :meth:`apply`
+
+        :param docs: a list of :class:`jina_pb2.Document` objects to work on; they could come from ``matches``/``chunks``.
+        :param context_doc: the owner of ``docs``
+        :param field: where ``docs`` comes from, either ``matches`` or ``chunks``
+        """
+
+    def __call__(self, *args, **kwargs):
+        self._traverse_apply(self.req.docs, *args, **kwargs)
+
+    def _traverse_apply(self, docs, *args, **kwargs):
+        """often useful when you delete a recursive structure """
+
+        def post_traverse(_docs, traverse_on, context_doc=None):
+            """
+            :param _docs: list of docs
+            :param traverse_on: "matches" or "chunks"
+            :param context_doc: the owner of ``_docs``, if None, then it is at the very top-level
+            :return:
+            """
+            if _docs:
+                for d in _docs:
+                    if d.level_depth < self._depth_end:
+                        post_traverse(getattr(d, traverse_on), traverse_on, d)
+                    if self.is_apply and (d.level_depth >= self._depth_start):
+                        self._apply(d, context_doc, traverse_on, *args, **kwargs)
+
+                # check first doc if in the required depth range
+                if self.is_apply_all and _docs[0].level_depth >= self._depth_start:
+                    self._apply_all(_docs, context_doc, traverse_on, *args, **kwargs)
+
+        def pre_traverse(_docs, traverse_on, context_doc=None):
+            if _docs:
+                # check first doc if in the required depth range
+                if self.is_apply_all and _docs[0].level_depth >= self._depth_start:
+                    self._apply_all(_docs, context_doc, traverse_on, *args, **kwargs)
+
+                for d in _docs:
+                    if self.is_apply and d.level_depth >= self._depth_start:
+                        self._apply(d, context_doc, traverse_on, *args, **kwargs)
+                    if d.level_depth < self._depth_end:
+                        pre_traverse(getattr(d, traverse_on), traverse_on, d)
+
+        if self.recursion_order == 'post':
+            _traverse = post_traverse
+        elif self.recursion_order == 'pre':
+            _traverse = pre_traverse
+        else:
+            raise ValueError(f'{self.recursion_order}')
+
+        if 'chunks' in self.traverse_fields:
+            _traverse(docs, 'chunks')
+        if 'matches' in self.traverse_fields:
+            for d in docs:
+                _traverse(d.matches, 'matches')
+
+
+class BaseExecutableDriver(BaseRecursiveDriver):
     """A :class:`BaseExecutableDriver` is an intermediate logic unit between the :class:`jina.peapods.pea.BasePea` and :class:`jina.executors.BaseExecutor`
         It reads the protobuf message, extracts/modifies the required information and then sends to the :class:`jina.executors.BaseExecutor`,
         finally it returns the message back to :class:`jina.peapods.pea.BasePea`.
@@ -259,7 +295,7 @@ class BaseExecutableDriver(BaseDriver):
         else:
             return lambda *args, **kwargs: None
 
-    def attach(self, executor: 'AnyExecutor', *args, **kwargs):
+    def attach(self, executor: 'AnyExecutor', *args, **kwargs) -> None:
         """Attach the driver to a :class:`jina.executors.BaseExecutor`"""
         super().attach(*args, **kwargs)
         if self._executor_name and isinstance(executor, CompoundExecutor):
@@ -279,7 +315,7 @@ class BaseExecutableDriver(BaseDriver):
         if self._method_name:
             self._exec_fn = getattr(self.exec, self._method_name)
 
-    def __getstate__(self):
+    def __getstate__(self) -> Dict[str, Any]:
         """Do not save the executor and executor function, as it would be cross-referencing and unserializable.
         In other words, a deserialized :class:`BaseExecutableDriver` from file is always unattached. """
         d = super().__getstate__()
