@@ -1,9 +1,14 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
+__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights re" \
+                "served."
+__license__ = "Apache-2.0"
+
 import os
 import pickle
 import re
+import subprocess
 import tempfile
 import uuid
 from datetime import datetime
@@ -17,7 +22,7 @@ from ruamel.yaml import StringIO
 from .decorators import as_train_method, as_update_method, store_init_kwargs
 from .metas import get_default_metas, fill_metas_with_defaults
 from ..excepts import EmptyExecutorYAML, BadWorkspace, BadPersistantFile, NoDriverForRequest, UnattachedDriver
-from ..helper import yaml, PathImporter, expand_dict, expand_env_var, valid_yaml_path
+from ..helper import yaml, PathImporter, expand_dict, expand_env_var, get_valid_local_config_source
 from ..logging.base import get_logger
 from ..logging.profile import TimeContext
 
@@ -132,6 +137,16 @@ class BaseExecutor(metaclass=ExecutorType):
         self._drivers = {}  # type: Dict[str, List['BaseDriver']]
         self._attached_pea = None
 
+    def _check_on_gpu(self):
+        if self.on_gpu:
+            try:
+                cuda_version = subprocess.check_output(['nvcc', '--version']).decode()
+                self.logger.success(f'CUDA compiler version: {cuda_version}')
+            except OSError:
+                self.logger.warning(
+                    'on_gpu=True, but you dont have CUDA compatible GPU, i will reset on_gpu=False ')
+                self.on_gpu = False
+
     def _post_init_wrapper(self, _metas: Dict = None, _requests: Dict = None, fill_in_metas: bool = True):
         with TimeContext('post initiating, this may take some time', self.logger):
             if fill_in_metas:
@@ -146,6 +161,7 @@ class BaseExecutor(metaclass=ExecutorType):
                 self._fill_requests(_requests)
 
             _before = set(list(vars(self).keys()))
+            self._check_on_gpu()
             self.post_init()
             self._post_init_vars = {k for k in vars(self) if k not in _before}
 
@@ -270,6 +286,9 @@ class BaseExecutor(metaclass=ExecutorType):
         del d['logger']
         for k in self._post_init_vars:
             del d[k]
+        cached = [k for k in d.keys() if k.startswith('CACHED_')]
+        for k in cached:
+            del d[k]
         return d
 
     def __setstate__(self, d):
@@ -357,7 +376,7 @@ class BaseExecutor(metaclass=ExecutorType):
         return True
 
     @classmethod
-    def load_config(cls: Type[AnyExecutor], filename: Union[str, TextIO], separated_workspace: bool = False,
+    def load_config(cls: Type[AnyExecutor], source: Union[str, TextIO], separated_workspace: bool = False,
                     replica_id: int = 0) -> AnyExecutor:
         """Build an executor from a YAML file.
 
@@ -367,10 +386,10 @@ class BaseExecutor(metaclass=ExecutorType):
         :param replica_id: the id of the storage of this replica, only effective when ``separated_workspace=True``
         :return: an executor object
         """
-        if not filename: raise FileNotFoundError
-        filename = valid_yaml_path(filename)
+        if not source: raise FileNotFoundError
+        source = get_valid_local_config_source(source)
         # first scan, find if external modules are specified
-        with (open(filename, encoding='utf8') if isinstance(filename, str) else filename) as fp:
+        with (open(source, encoding='utf8') if isinstance(source, str) else source) as fp:
             # ignore all lines start with ! because they could trigger the deserialization of that class
             safe_yml = '\n'.join(v if not re.match(r'^[\s-]*?!\b', v) else v.replace('!', '__tag: ') for v in fp)
             tmp = yaml.load(safe_yml)
@@ -386,7 +405,7 @@ class BaseExecutor(metaclass=ExecutorType):
                         mod = [mod]
 
                     if isinstance(mod, list):
-                        mod = [m if os.path.isabs(m) else os.path.join(os.path.dirname(filename), m) for m in mod]
+                        mod = [m if os.path.isabs(m) else os.path.join(os.path.dirname(source), m) for m in mod]
                         PathImporter.add_modules(*mod)
                     else:
                         raise TypeError(f'{type(mod)!r} is not acceptable, only str or list are acceptable')
@@ -395,7 +414,7 @@ class BaseExecutor(metaclass=ExecutorType):
                 tmp['metas']['replica_id'] = replica_id
 
             else:
-                raise EmptyExecutorYAML(f'{filename} is empty? nothing to read from there')
+                raise EmptyExecutorYAML(f'{source} is empty? nothing to read from there')
 
             tmp = expand_dict(tmp)
             stream = StringIO()
@@ -521,22 +540,6 @@ class BaseExecutor(metaclass=ExecutorType):
             r['metas'] = p
         return r
 
-    def add_driver(self, driver: 'BaseDriver', req_type: str):
-        """ Add a driver to this executor.
-
-        .. warning::
-            This has to be used *before* ``.attach(pea=self)`` to be effective
-
-        :param driver: the driver to add
-        :param req_type: the request type to handle by this driver
-        :return:
-        """
-        if not isinstance(driver, list):
-            driver = [driver]
-        if req_type not in self._drivers:
-            self._drivers[req_type] = []
-        self._drivers[req_type].extend(driver)
-
     def attach(self, *args, **kwargs):
         """Attach this executor to a :class:`jina.peapods.pea.BasePea`.
 
