@@ -1,22 +1,25 @@
 __copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
 __license__ = "Apache-2.0"
 
-import numbers
+__copyright__ = "Copyright (c) 2020 Jina AI Limited. All rights reserved."
+__license__ = "Apache-2.0"
 from typing import Union, Tuple, Dict
 
 import numpy as np
 
-from . import ImageChunkCrafter
+from .. import BaseCrafter
+from .helper import _load_image, _move_channel_axis, _resize_short
 
 
-class ImageResizer(ImageChunkCrafter):
+class ImageResizer(BaseCrafter):
     """
     :class:`ImageResizer` resize the image to the given size.
     """
 
     def __init__(self,
-                 target_size: Union[Tuple[int], int],
-                 how='BILINEAR',
+                 target_size: Union[Tuple[int, int], int],
+                 how: str = 'BILINEAR',
+                 channel_axis: int = -1,
                  *args, **kwargs):
         """
 
@@ -27,23 +30,21 @@ class ImageResizer(ImageChunkCrafter):
             Default is `BILINEAR`. Please refer to `PIL.Image` for detaisl.
         """
         super().__init__(*args, **kwargs)
-        if isinstance(target_size, numbers.Number):
+        if isinstance(target_size, int):
             self.output_dim = target_size
         else:
             raise ValueError(f'output_dim {target_size} should be an integer')
         self.how = how
+        self.channel_axis = channel_axis
 
-    def craft(self, blob: 'np.ndarray', chunk_id: int, doc_id: int, *args, **kwargs) -> Dict:
+    def craft(self, blob: 'np.ndarray', *args, **kwargs) -> Dict:
         """
         Resize the image array to the given size.
 
         :param blob: the ndarray of the image
-        :param chunk_id: the chunk id
-        :param doc_id: the doc id
         :return: a chunk dict with the cropped image
         """
-        raw_img = self.load_image(blob)
-        _img = self._resize_short(raw_img, self.output_dim, self.how)
-        img = self.restore_channel_axis(np.asarray(_img))
-        return dict(
-            doc_id=doc_id, offset=0, weight=1., blob=img.astype('float32'))
+        raw_img = _load_image(blob, self.channel_axis)
+        _img = _resize_short(raw_img, self.output_dim, self.how)
+        img = _move_channel_axis(np.asarray(_img), -1, self.channel_axis)
+        return dict(offset=0, weight=1., blob=img.astype('float32'))
