@@ -8,7 +8,7 @@ import numpy as np
 
 from .. import BaseExecutor
 from ..compound import CompoundExecutor
-from ...helper import call_obj_fn
+from ...helper import call_obj_fn, cached_property
 
 
 class BaseIndexer(BaseExecutor):
@@ -58,6 +58,26 @@ class BaseIndexer(BaseExecutor):
         :param keys: ``chunk_id`` in 1D-ndarray, shape B x 1
         :param vectors: vector representations in B x D
         """
+        raise NotImplementedError
+
+    def _validate_key_vector_shapes(self, keys, vectors):
+        if len(vectors.shape) != 2:
+            raise ValueError(f'vectors shape {vectors.shape} is not valid, expecting "vectors" to have rank of 2')
+
+        if not getattr(self, 'num_dim', None):
+            self.num_dim = vectors.shape[1]
+            self.dtype = vectors.dtype.name
+        elif self.num_dim != vectors.shape[1]:
+            raise ValueError(
+                f'vectors shape {vectors.shape} does not match with indexers\'s dim: {self.num_dim}')
+        elif self.dtype != vectors.dtype.name:
+            raise TypeError(
+                f'vectors\' dtype {vectors.dtype.name} does not match with indexers\'s dtype: {self.dtype}')
+        elif keys.shape[0] != vectors.shape[0]:
+            raise ValueError(f'number of key {keys.shape[0]} not equal to number of vectors {vectors.shape[0]}')
+        elif self.key_dtype != keys.dtype.name:
+            raise TypeError(
+                f'keys\' dtype {keys.dtype.name} does not match with indexers keys\'s dtype: {self.key_dtype}')
 
     def post_init(self):
         """query handler and write handler can not be serialized, thus they must be put into :func:`post_init`. """
@@ -65,10 +85,10 @@ class BaseIndexer(BaseExecutor):
         self._query_handler = None
         self._write_handler = None
 
-    def query(self, vectors: 'np.ndarray', top_k: int, *args, **kwargs) -> Tuple['np.ndarray', 'np.ndarray']:
+    def query(self, keys: 'np.ndarray', top_k: int, *args, **kwargs) -> Tuple['np.ndarray', 'np.ndarray']:
         """Find k-NN using query vectors, return chunk ids and chunk scores
 
-        :param vectors: query vectors in ndarray, shape B x D
+        :param keys: query vectors in ndarray, shape B x D
         :param top_k: int, the number of nearest neighbour to return
         :return: a tuple of two ndarray.
             The first is ids in shape B x K (`dtype=int`), the second is scores in shape B x K (`dtype=float`)
@@ -82,7 +102,7 @@ class BaseIndexer(BaseExecutor):
         """
         return self.get_file_from_workspace(self.index_filename)
 
-    @property
+    @cached_property
     def query_handler(self):
         """A readable and indexable object, could be dict, map, list, numpy array etc. """
         if self._query_handler is None and os.path.exists(self.index_abspath):
@@ -95,7 +115,7 @@ class BaseIndexer(BaseExecutor):
                                 'If you are querying data then the index file must be empty or broken.')
         return self._query_handler
 
-    @property
+    @cached_property
     def write_handler(self):
         """A writable and indexable object, could be dict, map, list, numpy array etc. """
 
@@ -163,7 +183,7 @@ class BaseKVIndexer(BaseIndexer):
     """
 
 
-class ChunkIndexer(CompoundExecutor):
+class CompoundIndexer(CompoundExecutor):
     """A Frequently used pattern for combining A :class:`BaseVectorIndexer` and :class:`BaseKVIndexer`.
     It will be equipped with predefined ``requests.on`` behaviors:
 
