@@ -7,20 +7,26 @@ import random
 import re
 import sys
 import time
+from argparse import ArgumentParser, Namespace
+from io import StringIO
 from itertools import islice
 from types import SimpleNamespace
-from typing import Iterator, Any, Union, List, Dict
-
+from typing import Tuple, Optional, Iterator, Any, Union, List, Dict, Set, TextIO
+import types
 import numpy as np
 from ruamel.yaml import YAML, nodes
 
 from . import JINA_GLOBAL
 
+if False:
+    from uvloop import Loop
+
 __all__ = ['batch_iterator', 'yaml',
            'load_contrib_module',
            'parse_arg',
            'PathImporter', 'random_port', 'get_random_identity', 'expand_env_var',
-           'colored', 'kwargs2list', 'valid_yaml_path']
+           'colored', 'kwargs2list', 'get_valid_local_config_source', 'valid_local_config_source',
+           'cached_property']
 
 
 def deprecated_alias(**aliases):
@@ -35,7 +41,7 @@ def deprecated_alias(**aliases):
     return deco
 
 
-def rename_kwargs(func_name, kwargs, aliases):
+def rename_kwargs(func_name: str, kwargs, aliases):
     from .logging import default_logger
     for alias, new in aliases.items():
         if alias in kwargs:
@@ -47,7 +53,7 @@ def rename_kwargs(func_name, kwargs, aliases):
             kwargs[new] = kwargs.pop(alias)
 
 
-def get_readable_size(num_bytes):
+def get_readable_size(num_bytes: int) -> str:
     if num_bytes < 1024:
         return f'{num_bytes} Bytes'
     elif num_bytes < 1024 ** 2:
@@ -58,7 +64,7 @@ def get_readable_size(num_bytes):
         return f'{num_bytes / (1024 ** 3):.1f} GB'
 
 
-def print_load_table(load_stat):
+def print_load_table(load_stat: Dict[str, List[Any]]):
     from .logging import default_logger
 
     load_table = []
@@ -73,7 +79,7 @@ def print_load_table(load_stat):
         default_logger.info('\n'.join(load_table))
 
 
-def print_load_csv_table(load_stat):
+def print_load_csv_table(load_stat: Dict[str, List[Any]]):
     from .logging import default_logger
 
     load_table = []
@@ -154,7 +160,7 @@ def _get_yaml():
     return y
 
 
-def parse_arg(v: str):
+def parse_arg(v: str) -> Union[bool, int, str]:
     if v.startswith('[') and v.endswith(']'):
         # function args must be immutable tuples not list
         tmp = v.replace('[', '').replace(']', '').strip().split(',')
@@ -178,24 +184,20 @@ def parse_arg(v: str):
     return v
 
 
-def countdown(t: int, logger=None, reason: str = 'I am blocking this thread'):
-    if not logger:
-        sys.stdout.write('\n')
-        sys.stdout.flush()
+def countdown(t: int, reason: str = 'I am blocking this thread') -> None:
+    sys.stdout.write('\n')
+    sys.stdout.flush()
     while t > 0:
         t -= 1
         msg = f'⏳ {colored("%3d" % t, "yellow")}s left: {reason}'
-        if logger:
-            logger.info(msg)
-        else:
-            sys.stdout.write(f'\r{msg}')
-            sys.stdout.flush()
+        sys.stdout.write(f'\r{msg}')
+        sys.stdout.flush()
         time.sleep(1)
     sys.stdout.write('\n')
     sys.stdout.flush()
 
 
-def load_contrib_module():
+def load_contrib_module() -> List[Any]:
     if 'JINA_CONTRIB_MODULE_IS_LOADING' not in os.environ:
 
         contrib = os.getenv('JINA_CONTRIB_MODULE')
@@ -217,28 +219,28 @@ def load_contrib_module():
 class PathImporter:
 
     @staticmethod
-    def _get_module_name(absolute_path):
+    def _get_module_name(absolute_path: str) -> str:
         module_name = os.path.basename(absolute_path)
         module_name = module_name.replace('.py', '')
         return module_name
 
     @staticmethod
-    def add_modules(*paths):
+    def add_modules(*paths) -> 'types.ModuleType':
         for p in paths:
             if not os.path.exists(p):
                 raise FileNotFoundError('cannot import module from %s, file not exist', p)
-            module, spec = PathImporter._path_import(p)
+            module = PathImporter._path_import(p)
         return module
 
     @staticmethod
-    def _path_import(absolute_path):
+    def _path_import(absolute_path: str) -> 'types.ModuleType':
         import importlib.util
         module_name = PathImporter._get_module_name(absolute_path)
         spec = importlib.util.spec_from_file_location(module_name, absolute_path)
         module = importlib.util.module_from_spec(spec)
         spec.loader.exec_module(module)
         sys.modules[spec.name] = module
-        return module, spec
+        return module
 
 
 _random_names = (('first', 'great', 'local', 'small', 'right', 'large', 'young', 'early', 'major', 'clear', 'black',
@@ -258,14 +260,32 @@ def random_name() -> str:
 
 
 def random_port() -> int:
+    import threading
     from contextlib import closing
     import socket
-    import threading
-    with threading.Lock():
-        with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
-            s.bind(('', 0))
-            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-            return s.getsockname()[1]
+
+    def _get_port(port=0):
+        _p = None
+        with threading.Lock():
+            with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
+                try:
+                    s.bind(('', port))
+                    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+                    _p = s.getsockname()[1]
+                except socket.error:
+                    pass
+        return _p
+
+    _port = None
+    if 'JINA_RANDOM_PORTS' in os.environ:
+        min_port, max_port = 49152, 65535
+        while True:
+            _port = random.randrange(min_port, max_port)
+            if _get_port(_port) is not None:
+                break
+    else:
+        _port = _get_port()
+    return _port
 
 
 def get_registered_ports(stack_id: int = JINA_GLOBAL.stack.id):
@@ -412,11 +432,50 @@ _COLORS = {
 
 _RESET = '\033[0m'
 
+
+def build_url_regex_pattern():
+    ul = '\u00a1-\uffff'  # Unicode letters range (must not be a raw string).
+
+    # IP patterns
+    ipv4_re = r'(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)(?:\.(?:25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}'
+    ipv6_re = r'\[[0-9a-f:.]+\]'  # (simple regex, validated later)
+
+    # Host patterns
+    hostname_re = r'[a-z' + ul + r'0-9](?:[a-z' + ul + r'0-9-]{0,61}[a-z' + ul + r'0-9])?'
+    # Max length for domain name labels is 63 characters per RFC 1034 sec. 3.1
+    domain_re = r'(?:\.(?!-)[a-z' + ul + r'0-9-]{1,63}(?<!-))*'
+    tld_re = (
+            r'\.'  # dot
+            r'(?!-)'  # can't start with a dash
+            r'(?:[a-z' + ul + '-]{2,63}'  # domain label
+                              r'|xn--[a-z0-9]{1,59})'  # or punycode label
+                              r'(?<!-)'  # can't end with a dash
+                              r'\.?'  # may have a trailing dot
+    )
+    host_re = '(' + hostname_re + domain_re + tld_re + '|localhost)'
+
+    return re.compile(
+        r'^(?:[a-z0-9.+-]*)://'  # scheme is validated separately
+        r'(?:[^\s:@/]+(?::[^\s:@/]*)?@)?'  # user:pass authentication
+        r'(?:' + ipv4_re + '|' + ipv6_re + '|' + host_re + ')'
+                                                           r'(?::\d{2,5})?'  # port
+                                                           r'(?:[/?#][^\s]*)?'  # resource path
+                                                           r'\Z', re.IGNORECASE)
+
+
+url_pat = build_url_regex_pattern()
+
+
+def is_url(text):
+    return url_pat.match(text) is not None
+
+
 if os.name == 'nt':
     os.system('color')
 
 
-def colored(text, color=None, on_color=None, attrs=None):
+def colored(text: str, color: Optional[str] = None,
+            on_color: Optional[str] = None, attrs: Union[str, list, None] = None) -> str:
     if 'JINA_LOG_NO_COLOR' not in os.environ:
         fmt_str = '\033[%dm%s'
         if color:
@@ -454,7 +513,7 @@ def get_tags_from_node(node) -> List[str]:
     return list(set(list(node_recurse_generator(node))))
 
 
-def kwargs2list(kwargs: Dict):
+def kwargs2list(kwargs: Dict) -> List[str]:
     args = []
     for k, v in kwargs.items():
         k = k.replace('_', '-')
@@ -469,7 +528,7 @@ def kwargs2list(kwargs: Dict):
     return args
 
 
-def valid_yaml_path(path: str, to_stream: bool = False):
+def get_valid_local_config_source(path: str, to_stream: bool = False) -> Union[StringIO, TextIO, str]:
     # priority, filepath > classname > default
     import io
     from pkg_resources import resource_filename
@@ -486,6 +545,7 @@ def valid_yaml_path(path: str, to_stream: bool = False):
         return resource_filename('jina', '/'.join(('resources', 'executors.%s.yml' % path)))
     elif path.startswith('!'):
         # possible YAML content
+        path = path.replace('|', '\n    with: ')
         return io.StringIO(path)
     elif path.startswith('- !'):
         # possible driver YAML content, right now it is only used for debugging
@@ -503,7 +563,16 @@ def valid_yaml_path(path: str, to_stream: bool = False):
                                 ' or a valid file path, or a supported class name.' % path)
 
 
-def get_parsed_args(kwargs, parser, parser_name: str = None):
+def valid_local_config_source(path: str) -> bool:
+    try:
+        get_valid_local_config_source(path)
+        return True
+    except:
+        return False
+
+
+def get_parsed_args(kwargs: Dict[str, Union[str, int, bool]], parser: ArgumentParser, parser_name: str = None) -> Union[
+    Tuple[List[str], Namespace, List[Any]], Tuple[List[str], Namespace, List[str]]]:
     args = kwargs2list(kwargs)
     try:
         p_args, unknown_args = parser.parse_known_args(args)
@@ -520,7 +589,7 @@ def get_parsed_args(kwargs, parser, parser_name: str = None):
     return args, p_args, unknown_args
 
 
-def get_non_defaults_args(args, parser, taboo=(None,)) -> Dict:
+def get_non_defaults_args(args: Namespace, parser: ArgumentParser, taboo: Set[Optional[str]] = (None,)) -> Dict:
     non_defaults = {}
     _defaults = vars(parser.parse_args([]))
     for k, v in vars(args).items():
@@ -529,7 +598,7 @@ def get_non_defaults_args(args, parser, taboo=(None,)) -> Dict:
     return non_defaults
 
 
-def get_full_version():
+def get_full_version() -> str:
     from . import __version__, __proto_version__, __jina_env__
     from google.protobuf.internal import api_implementation
     import os, zmq, numpy, google.protobuf, grpc, ruamel.yaml
@@ -564,31 +633,47 @@ def get_full_version():
         default_logger.exception(e)
 
 
-def is_url(text):
-    url_pat = re.compile(
-        r'^(?:http|ftp)s?://'  # http:// or https://
-        r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
-        r'localhost|'  # localhost...
-        r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
-        r'(?::\d+)?'  # optional port
-        r'(?:/?|[/?]\S+)$', re.IGNORECASE)
-    return url_pat.match(text) is not None
-
-
 def use_uvloop():
-    if 'JINA_UVLOOP' in os.environ:
+    if 'JINA_DISABLE_UVLOOP' not in os.environ:
         try:
             import asyncio
             import uvloop
             asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
         except (ModuleNotFoundError, ImportError):
             from .logging import default_logger
-            default_logger.error('JINA_UVLOOP is enabled but you did not install uvloop. Try "pip install uvloop"')
+            default_logger.error(
+                'Since v0.3.6 jina uses uvloop to manage events and sockets, it often yields 20% speedup'
+                'you did not install uvloop. Try "pip install uvloop"')
 
 
-def is_uvloop_used(loop=None):
+def show_ioloop_backend(loop: Optional['Loop'] = None) -> None:
     if loop is None:
         import asyncio
         loop = asyncio.get_event_loop()
     from .logging import default_logger
     default_logger.info(f'using {loop.__class__} as event loop')
+
+
+def rsetattr(obj, attr: str, val):
+    pre, _, post = attr.rpartition('.')
+    return setattr(rgetattr(obj, pre) if pre else obj, post, val)
+
+
+def rgetattr(obj, attr: str, *args):
+    def _getattr(obj, attr):
+        return getattr(obj, attr, *args)
+
+    return functools.reduce(_getattr, [obj] + attr.split('.'))
+
+
+class cached_property:
+    def __init__(self, func):
+        self.func = func
+
+    def __get__(self, obj, cls):
+        cached_value = obj.__dict__.get(f'CACHED_{self.func.__name__}', None)
+        if cached_value is not None:
+            return cached_value
+
+        value = obj.__dict__[f'CACHED_{self.func.__name__}'] = self.func(obj)
+        return value
