@@ -77,10 +77,10 @@ def hello_world(args):
 
 
 def flow(args):
-    """Start a Flow from a YAML file"""
+    """Start a Flow from a YAML file or a docker image"""
     from ..flow import Flow
     if args.yaml_path:
-        f = Flow.load_config(args.yaml_path)
+        f = Flow.load_config(args.uses)
         f._update_args(args)
         with f:
             f.block()
