@@ -117,11 +117,11 @@ def set_hw_parser(parser=None):
     gp.add_argument('--shards', type=int,
                     default=2,
                     help='number of shards when index and query')
-    gp.add_argument('--replicas', type=int,
+    gp.add_argument('--parallel', type=int,
                     default=2,
-                    help='number of replicas when index and query')
+                    help='number of parallel when index and query')
     gp = add_arg_group(parser, 'index arguments')
-    gp.add_argument('--index-yaml-path', type=str,
+    gp.add_argument('--index-uses', type=str,
                     default=resource_filename('jina', '/'.join(('resources', 'helloworld.flow.index.yml'))),
                     help='the yaml path of the index flow')
     gp.add_argument('--index-data-url', type=str,
@@ -131,7 +131,7 @@ def set_hw_parser(parser=None):
                     default=1024,
                     help='the batch size in indexing')
     gp = add_arg_group(parser, 'query arguments')
-    gp.add_argument('--query-yaml-path', type=str,
+    gp.add_argument('--query-uses', type=str,
                     default=resource_filename('jina', '/'.join(('resources', 'helloworld.flow.query.yml'))),
                     help='the yaml path of the query flow')
     gp.add_argument('--query-data-url', type=str,
@@ -170,7 +170,6 @@ def set_flow_parser(parser=None):
                     help='type of the output')
     gp.add_argument('--output-path', type=argparse.FileType('w', encoding='utf8'),
                     help='output path of the flow')
-
     return parser
 
 
@@ -182,28 +181,26 @@ def set_pea_parser(parser=None):
     show_all = 'JINA_FULL_CLI' in os.environ
     if not parser:
         parser = set_base_parser()
-
     gp0 = add_arg_group(parser, 'pea basic arguments')
     gp0.add_argument('--name', type=str,
                      help='the name of this pea, used to identify the pod and its logs.')
     gp0.add_argument('--identity', type=str, default=get_random_identity(),
                      help='the identity of the sockets, default a random string')
-    gp0.add_argument('--yaml-path', type=str, default='BaseExecutor',
-                     help='the yaml config of the executor, it could be '
+    gp0.add_argument('--uses', type=str, default='_pass',
+                     help='the config of the executor, it could be '
                           '> a YAML file path, '
                           '> a supported executor\'s class name, '
-                          '> one of "_clear", "_route", "_forward", "_logforward", "_merge" '
-                          '> the content of YAML config (must starts with "!")')  # pod(no use) -> pea
+                          '> one of "_clear", "_route", "_pass", "_logforward", "_merge" '
+                          '> the content of YAML config (must starts with "!")'
+                          '> a docker image')  # pod(no use) -> pea
     gp0.add_argument('--py-modules', type=str, nargs='*',
                      help='the customized python modules need to be imported before loading the'
                           ' executor')
 
     gp1 = add_arg_group(parser, 'pea container arguments')
-    gp1.add_argument('--image', type=str,
-                     help='the name of the docker image that this pea runs with. it is also served as an indicator '
-                          'of containerization. '
-                          'when this and --yaml-path are both given then the docker image '
-                          'is used and its original yaml configuration is replaced by the given --yaml-path')
+    gp1.add_argument('--uses-internal', type=str, default='BaseExecutor',
+                     help='The executor config that is passed to the docker image if a docker image is used in uses. '
+                          'It cannot be another docker image ')
     gp1.add_argument('--entrypoint', type=str,
                      help='the entrypoint command overrides the ENTRYPOINT in docker image. '
                           'when not set then the docker image ENTRYPOINT takes effective.')
@@ -251,7 +248,7 @@ def set_pea_parser(parser=None):
                           'dump_interval will be ignored')
     gp3.add_argument('--separated-workspace', action='store_true', default=False,
                      help='the data and config files are separated for each pea in this pod, '
-                          'only effective when BasePod\'s `replicas` > 1')
+                          'only effective when BasePod\'s `parallel` > 1')
     gp3.add_argument('--replica-id', type=int, default=-1,
                      help='the id of the storage of this replica, only effective when `separated_workspace=True`')
 
@@ -311,8 +308,8 @@ def set_pod_parser(parser=None):
     set_pea_parser(parser)
 
     gp4 = add_arg_group(parser, 'pod replica arguments')
-    gp4.add_argument('--replicas', type=int, default=1,
-                     help='number of parallel peas in the pod running at the same time (i.e. replicas), '
+    gp4.add_argument('--parallel', '--shards', type=int, default=1,
+                     help='number of parallel peas in the pod running at the same time, '
                           '`port_in` and `port_out` will be set to random, '
                           'and routers will be added automatically when necessary')
     gp4.add_argument('--polling', type=PollingType.from_string, choices=list(PollingType),
@@ -322,9 +319,9 @@ def set_pod_parser(parser=None):
     gp4.add_argument('--scheduling', type=SchedulerType.from_string, choices=list(SchedulerType),
                      default=SchedulerType.LOAD_BALANCE,
                      help='the strategy of scheduling workload among peas')
-    gp4.add_argument('--reducing-yaml-path', type=str, default='_merge',
-                     help='the executor used for reducing the result from all replicas, '
-                          'accepted type follows "--yaml-path"')
+    gp4.add_argument('--uses-reducing', type=str, default='_merge',
+                     help='the executor used for reducing the result from all parallel, '
+                          'accepted type follows "--uses"')
     gp4.add_argument('--shutdown-idle', action='store_true', default=False,
                      help='shutdown this pod when all peas are idle')
 
@@ -480,13 +477,15 @@ def set_client_cli_parser(parser=None):
                      help='the starting number of request id, the consequent request_id will increment by one')
     _gp.add_argument('--first-doc-id', type=int,
                      default=0,
-                     help='the starting number of doc_id, the consequent doc_id will increment by one')
+                     help='the starting number of the first doc, the consequent id will increment by one')
     _gp.add_argument('--random-doc-id', action='store_true', default=False,
-                     help='randomize the doc_id, if this is set then `first_request_id` is ignored')
+                     help='randomize document id, if this is set then `first_request_id` is ignored')
     gp1.add_argument('--timeout-ready', type=int, default=10000,
                      help='timeout (ms) of a pea is ready for request, -1 for waiting forever')
     gp1.add_argument('--filter-by', type=str, nargs='*',
                      help='field names to search on')
+    gp1.add_argument('--skip-dry-run', action='store_true', default=False,
+                     help='skip dry run (connectivity test) before sending every request')
     return parser
 
 
